function testWordList_GUI

close all

% ==== Control variable/parameters for the experiment ====
% retrieve words list
exp = open('wordsEncoding.mat');

% define index numbers for trials
nTotal = length(exp.wordsEncoding);
currentTrial = 1;

% this is for the randomization
nRandIdx = randperm(nTotal);

% store subjective judgments in responseValues
% store response time in responseTime
responseValues = zeros(1, nTotal);
responseTime = zeros(1, nTotal);


% import the beep sound to indicate the stimuli presentation
[beep, fs] = wavread('beep.wav');

% waitTime is the ramdomized waiting time before the beep sound
% for the first trial, wait for 2 sec after closing the message box.
waitTime = 2;
% beepTime is the gap between beep and word display
beepTime = 0.5;
% displayTime is the duration of the word display
displayTime = 1;


% ======= Define and initialize GUI ======
% Reference: MATLAB HELP - Creating Graphical User Interfaces - Creating
% GUI Programmatically.
% 'Position' handle: [distance from left, distance from bottom, width, height].

% === set the main figure
f = figure('MenuBar', 'none', 'NumberTitle', 'off', 'Visible','off','Position',[360,500,450,300]);

% color matching
defaultBackground = get(0,'defaultUicontrolBackgroundColor');
set(f,'Color',defaultBackground)
% Assign the GUI a name to appear in the window title.
set(f,'Name','Listening test');
% Move the GUI to the center of the screen.
movegui(f,'center');

% === set the additional components
% set trial number indication
htext = uicontrol('Style','text',...
    'String',sprintf('Trial %d of %d', currentTrial, nTotal), ...
    'Units','normalized','Position',[.25, .8, .5, .1], 'FontSize', 14 ,...
    'HorizontalAlignment', 'center');
% set text field for the word list
hword = uicontrol('Style','text', 'BackgroundColor', 'white', ...
    'String', [],  'HorizontalAlignment', 'center', ...
    'Units','normalized','Position',[.4, .6, .2, .1], 'FontSize', 20  );

% set push buttons - "next"
hnext = uicontrol('Style','pushbutton',...
    'String','Next','Units','normalized','Position',[.7, .1, .2, .1],...
    'Callback',{@nextbutton_Callback}, 'FontSize', 14 );

% set two exclusive radio buttons (yes/no) using buttongroup
hbuttons = uibuttongroup('Parent', f, 'Position',[.25 .25 .5 .3], ...
    'Title', 'Is it an animal?',  'FontSize', 14 );
buttonYes = uicontrol('Style','radiobutton','String','Yes',...
    'Units','normalized','pos',[.2 .3 .25 .5], 'FontSize', 12 ,'Parent', hbuttons, ...
    'HandleVisibility','off');
buttonNo = uicontrol('Style','radiobutton','String','No', ...
    'Units','normalized','pos',[.55 .3 .25 .5], 'FontSize', 12 ,'Parent', hbuttons, ...
    'HandleVisibility','off');


% Initialize button group properties.
set(hbuttons,'SelectionChangeFcn',@buttons_Callback); % define cbk function
set(hbuttons,'SelectedObject',[]);  % No selection


% === define timer objects

tBeep = timer('TimerFcn', @tBeep_Callback,'StartDelay', waitTime);
tDisplay = timer('TimerFcn', @tDisplay_Callback,'StartDelay', waitTime + beepTime);
tClear = timer('TimerFcn', @tClear_Callback, 'StartDelay', waitTime + beepTime + displayTime);


% === Make the GUI visible when the subject is ready
uiwait(msgbox('Are you ready for the experiment?'));
set(f,'Visible','on');

% === responsetime measurement w/ the stopwatch
t = 0;
% === beep, display the word, then clear the word
start([tBeep, tDisplay, tClear]);


% ======= Callback Functions =======
% subroutines to make actions according to the button input

    function nextbutton_Callback(source,eventdata)
        % retrieve the response, store, clear the input field,
        % then proceed to the next trial.

        % %% todo later: divide trials into practice and test session
        % %% randomization should take place here

        % retrieve and store the response time
        %t = toc;
        responseTime(nRandIdx(currentTrial)) = t;

        % Retrieve the response (yes or no)
        response = (get(get(hbuttons,'SelectedObject'),'String'));

        % in case the subject fail to make a choice
        if isempty(response)
            response = 'Fail';
        end

        if  response(1) == 'Y'
            responseValues(nRandIdx(currentTrial)) = 1;
        elseif response(1) == 'N'
            responseValues(nRandIdx(currentTrial)) = 0;
        elseif response(1) == 'F'
            responseValues(nRandIdx(currentTrial)) = NaN;
        end

        % clear the selection of the buttons
        set(hbuttons,'SelectedObject',[]);  % No selection for radio buttons

        % quit or proceed to the next trial
        if currentTrial == nTotal
            % to quit
            save resultsEncoding.mat responseValues  responseTime
            msgbox('End of experiment - Thank you!')
            close(f);
            delete(tDisplay);
            delete(tBeep);
            delete(tClear);
        else
            stop([tBeep, tDisplay, tClear]);
            % count up for the next trial number
            currentTrial = currentTrial+1;
            % display the next trial number in the text field
            set(htext, 'String',sprintf('Trial %d of %d', currentTrial, nTotal));
            % set waitTime randomized between 1.5-3 sec.
            waitTime = (round(1.5*rand(1) *100))*0.01 + 1.5;
            % reset timers with the new randomized wait time
            set(tBeep,'StartDelay', waitTime);
            set(tDisplay,'StartDelay', waitTime + beepTime);
            set(tClear,'StartDelay', waitTime + beepTime + displayTime);
            start([tBeep, tDisplay, tClear]);
        end
    end


    function buttons_Callback(source,eventdata)
        % store the response time
        t = toc - beepTime;
    end

    function tBeep_Callback(source, eventdata)
        % play sound
        sound(beep,fs);
    end
    function tDisplay_Callback(source, eventdata)
        % trigger the stopwatch, then word to appear
        tic;
        set(hword, 'String', exp.wordsEncoding(nRandIdx(currentTrial)));
    end
    function tClear_Callback(source, eventdata)
        % clear the text field
        set(hword, 'String',[]);,
    end

end